let prec (c: char) : int =
  match c with
  | 'a' -> 5
  | '&' -> 4
  | '|' -> 3
  | '>' -> 2
  | _   -> 1

let find_op_out_parent (s: string) (i: int) (j: int) =
  let rec aux (i: int) (p: int) =
    if i > j then []
    else
      match s.[i], p with
      | '&', 0 -> ('&', i) :: (aux (i+1) p)
      | '|', 0 -> ('|', i) :: (aux (i+1) p)
      | '>', 0 -> ('>', i) :: (aux (i+1) p)
      | '=', 0 -> ('=', i) :: (aux (i+1) p)
      | '(', _ -> aux (i+1) (p+1)
      | ')', _ -> aux (i+1) (p-1)
      | _, _ -> aux (i+1) p
  in
  List.fold_left (fun (op, i) (op', i') -> if prec op < prec op' then (op, i) else (op', i')) ('a', -1) (aux i 0)

let rec all_var_char (s: string) (i: int) (j: int) =
  (i > j) ||
  ((int_of_char 'a' <= int_of_char (s.[i]) && int_of_char (s.[i]) <= int_of_char 'z')
   && all_var_char s (i+1) j)

let rec reader (s: string) (i: int) (j: int) =
  if s.[i] = ' ' then
    reader s (i+1) j
  else if s.[j] = ' ' then
    reader s i (j-1)
  else
    let (op, k) = find_op_out_parent s i j in
    if k > 0 then
      match op with
      | '&' -> Conj(reader s i (k-1), reader s (k+1) (j))
      | '|' -> Disj(reader s i (k-1), reader s (k+1) (j))
      | '>' -> Impl(reader s i (k-1), reader s (k+1) (j))
      | _   -> Equiv(reader s i (k-1), reader s (k+1) (j))
    else
      match s.[i], j-i with
      | '~', _ -> Neg(reader s (i+1) j)
      | 'T', 0 -> Vrai
      | 'B', 0 -> Faux
      | '(', _ -> reader s (i+1) (j-1)
      | _ -> (assert (all_var_char s i j) ; Var (String.sub s i (j-i+1)))


let parse (s: string) =
  reader s 0 ((String.length s) - 1)

